@@ -2,12 +2,16 @@
 #
 
 errorsarlm <- function(formula, data = list(), listw, na.action=na.fail, 
-	method="eigen", quiet=TRUE, zero.policy=FALSE, tol.solve=1.0e-10, 
-        tol.opt=.Machine$double.eps^0.5, sparsedebug=FALSE) {
+	method="eigen", quiet=TRUE, zero.policy=FALSE, interval=c(-1,0.999), 
+	tol.solve=1.0e-10, tol.opt=.Machine$double.eps^0.5, control, 
+	optim=FALSE, sparsedebug=FALSE) {
 	mt <- terms(formula, data = data)
 	mf <- lm(formula, data, na.action=na.action, method="model.frame")
 	na.act <- attr(mf, "na.action")
 	if (!inherits(listw, "listw")) stop("No neighbourhood list")
+	can.sim <- as.logical(NA)
+	if (listw$style %in% c("W", "S")) 
+		can.sim <- can.be.simmed(listw)
 	if (!is.null(na.act)) {
 	    subset <- !(1:length(listw$neighbours) %in% na.act)
 	    listw <- subset(listw, subset, zero.policy=zero.policy)
@@ -16,7 +20,15 @@ errorsarlm <- function(formula, data = list(), listw, na.action=na.fail,
 		"Jacobian calculated using "))
 	switch(method,
 		eigen = if (!quiet) cat("neighbourhood matrix eigenvalues\n"),
-		sparse = if (!quiet) cat("sparse matrix techniques\n"),
+		sparse = {if (!quiet) cat("sparse matrix techniques\n")},
+	        SparseM = {
+		    if (listw$style %in% c("W", "S") && !can.sim)
+		    stop("SparseM method requires symmetric weights")
+		    if (listw$style %in% c("B", "C", "U") && 
+			!(is.symmetric.glist(listw$neighbours, listw$weights)))
+		    stop("SparseM method requires symmetric weights")
+		    if (!quiet) cat("sparse matrix techniques using SparseM\n")
+		},
 		stop("...\n\nUnknown method\n"))
 	y <- model.response(mf, "numeric")
 	if (any(is.na(y))) stop("NAs in dependent variable")
@@ -49,25 +61,86 @@ errorsarlm <- function(formula, data = list(), listw, na.action=na.fail,
 	}
 	colnames(WX) <- xcolnames
 	rm(wx)
+	similar <- FALSE
+	lm.rho <- lm.fit(cbind(x, wy), y)
+	rho <- coef(lm.rho)[length(coef(lm.rho))]
+	if ((optim) && missing(control)) {
+		control<- list(trace=0, fnscale=-1, factr=tol.opt,
+			pgtol=tol.opt)
+	}
 	if (method == "eigen") {
 		if (!quiet) cat("Computing eigenvalues ...\n")
-		eig <- eigenw(listw)
+		if (listw$style %in% c("W", "S") & can.sim) {
+			eig <- eigenw(similar.listw(listw))
+			similar <- TRUE
+		} else eig <- eigenw(listw)
 		if (!quiet) cat("\n")
 # range inverted 031031, email from Salvati Nicola (and Rein Halbersma)
 		if (is.complex(eig)) eig.range <- 1/range(Re(eig))
 		else eig.range <- 1/range(eig)
-		opt <- optimize(sar.error.f, interval=eig.range, maximum=TRUE,
-			tol=tol.opt, eig=eig,
-			y=y, wy=wy, x=x, WX=WX, n=n, quiet=quiet)
-	} else {
+		if (rho < eig.range[1]+.Machine$double.eps) rho <- 0.0
+		if (rho > eig.range[2]-.Machine$double.eps) rho <- 0.0
+		if (optim) {
+		    opt <- optim(par=c(rho), sar.error.f, method="L-BFGS-B", 
+			lower=eig.range[1]+.Machine$double.eps, 
+			upper=eig.range[2]-.Machine$double.eps, 
+			control=control, 
+			eig=eig, y=y, wy=wy, x=x, WX=WX, n=n, quiet=quiet)
+		    if (opt$convergence == 1) warning("iteration limit reached")
+		    if (opt$convergence == 51) warning(opt$message)
+		    if (opt$convergence == 52) warning(opt$message)
+		    lambda <- c(opt$par[1])
+		    names(lambda) <- "lambda"
+		    LL <- c(opt$value)
+		} else {
+		    opt <- optimize(sar.error.f, 
+			lower=eig.range[1]+.Machine$double.eps, 
+			upper=eig.range[2]-.Machine$double.eps, maximum=TRUE,
+			tol=tol.opt, eig=eig, y=y, wy=wy, x=x, WX=WX, 
+			n=n, quiet=quiet)
+		    lambda <- opt$maximum
+		    names(lambda) <- "lambda"
+		    LL <- opt$objective
+		}
+	} else if (method == "sparse") {
 		sn <- listw2sn(listw)
-		opt <- optimize(sar.error.f.s, interval=c(-1,1), maximum=TRUE,
+		opt <- optimize(sar.error.f.s, interval=interval, maximum=TRUE,
 			tol=tol.opt, sn=sn, y=y, wy=wy, x=x, WX=WX, 
 			n=n, quiet=quiet, sparsedebug=sparsedebug)
+			warning("The sparse method will be withdrawn shortly")
+		lambda <- opt$maximum
+		names(lambda) <- "lambda"
+		LL <- opt$objective
+	} else if (method == "SparseM") {
+		if (listw$style %in% c("W", "S") & can.sim) {
+			csrw <- asMatrixCsrListw(similar.listw(listw))
+			similar <- TRUE
+		} else csrw <- asMatrixCsrListw(listw)
+		I <- asMatrixCsrI(n)
+		if (rho <= interval[1]) rho <- 0.0
+		if (rho >= interval[2]) rho <- 0.0
+		if (optim) {
+		    opt <- optim(par=c(rho), sar.error.f.sM, method="L-BFGS-B", 
+			lower=interval[1],
+			upper=interval[2],
+			control=control, 
+			csrw=csrw, I=I, y=y, wy=wy, x=x, WX=WX, 
+			n=n, quiet=quiet)
+		    if (opt$convergence == 1) warning("iteration limit reached")
+		    if (opt$convergence == 51) warning(opt$message)
+		    if (opt$convergence == 52) warning(opt$message)
+		    lambda <- c(opt$par[1])
+		    names(lambda) <- "lambda"
+		    LL <- c(opt$value)
+		} else {
+		    opt <- optimize(sar.error.f.sM, interval=interval, 
+			maximum=TRUE, tol=tol.opt, csrw=csrw, I=I, y=y, wy=wy, 
+			x=x, WX=WX, n=n, quiet=quiet)
+		    lambda <- opt$maximum
+		    names(lambda) <- "lambda"
+		    LL <- opt$objective
+		}
 	}
-	lambda <- opt$maximum
-	names(lambda) <- "lambda"
-	LL <- opt$objective
 	lm.target <- lm(I(y - lambda*wy) ~ I(x - lambda*WX) - 1)
 	r <- as.vector(residuals(lm.target))
 	fit <- as.vector(y - r)
@@ -85,7 +158,7 @@ errorsarlm <- function(formula, data = list(), listw, na.action=na.fail,
 	if (method == "eigen") {
 		tr <- function(A) sum(diag(A))
 		W <- listw2mat(listw)
-		A <- solve(diag(n) - lambda*W, tol=tol.solve)
+		A <- solve(diag(n) - lambda*W)
 		WA <- W %*% A
 		asyvar <- matrix(0, nrow=2+p, ncol=2+p)
 		asyvar[1,1] <- n / (2*(s2^2))
@@ -103,12 +176,12 @@ errorsarlm <- function(formula, data = list(), listw, na.action=na.fail,
 	call <- match.call()
 	names(r) <- names(y)
 	names(fit) <- names(y)
-	ret <- structure(list(type="error", lambda=lambda, 
+	ret <- structure(list(type="error", lambda=lambda,
 		coefficients=coef.lambda, rest.se=rest.se, 
 		LL=LL, s2=s2, SSE=SSE, parameters=(m+2), lm.model=lm.model, 
 		method=method, call=call, residuals=r, lm.target=lm.target,
-		fitted.values=fit, ase=ase, formula=formula,
-		se.fit=NULL, resvar=asyvar1,
+		opt=opt, fitted.values=fit, ase=ase, formula=formula,
+		se.fit=NULL, resvar=asyvar1, similar=similar,
 		lambda.se=lambda.se, LMtest=LMtest, zero.policy=zero.policy), 
 		class=c("sarlm"))
 	if (zero.policy) {
@@ -133,7 +206,7 @@ sar.error.f <- function(lambda, eig, y, wy, x, WX, n, quiet)
 	if (is.complex(eig)) det <- Re(prod(1 - lambda*eig)) 
 	else det <- prod(1 - lambda*eig)
 	ret <- (log(det) - ((n/2)*log(2*pi)) - (n/2)*log(s2) - (1/(2*(s2)))*SSE)
-	if (!quiet) cat("Lambda:\t", lambda, "\tfunction value:\t", ret, "\n")
+	if (!quiet) cat("(eigen) lambda:\t", lambda, "\tfunction value:\t", ret, "\n")
 	ret
 }
 
@@ -147,7 +220,20 @@ sar.error.f.s <- function(lambda, sn, y, wy, x, WX, n, quiet, sparsedebug)
 	s2 <- SSE/n
 	ret <- (logSpwdet(sparseweights=sn, rho=lambda, debug=sparsedebug) - 
 		((n/2)*log(2*pi)) - (n/2)*log(s2) - (1/(2*(s2)))*SSE)
-	if (!quiet) cat("Lambda:\t", lambda, "\tfunction value:\t", ret, "\n")
+	if (!quiet) cat("(sparse) lambda:\t", lambda, "\tfunction value:\t", ret, "\n")
+	ret
+}
+
+sar.error.f.sM <- function(lambda, csrw, I, y, wy, x, WX, n, quiet) {
+	yl <- y - lambda*wy
+	xl <- x - lambda*WX
+	xl.q <- qr.Q(qr(xl))
+	xl.q.yl <- t(xl.q) %*% yl
+	SSE <- t(yl) %*% yl - t(xl.q.yl) %*% xl.q.yl
+	s2 <- SSE/n
+	ret <- (log(det(I - lambda * csrw)) -
+		((n/2)*log(2*pi)) - (n/2)*log(s2) - (1/(2*(s2)))*SSE)
+	if (!quiet) cat("(SparseM) lambda:\t", lambda, "\tfunction value:\t", ret, "\n")
 	ret
 }
 
